from django.test import TestCase
from tour.urls import urlpatterns


class UrlsTest(TestCase):
    def test_that_urls_are_defined(self):
        """
        Should have several urls defined.
        """
        self.assertEqual(len(urlpatterns), 1)



from django.conf import settings
from django.db import models
from django.utils.module_loading import import_by_path
from manager_utils import ManagerUtilsManager
import six


class TourManager(ManagerUtilsManager):
    """
    Provides extra functionality for the Tour model
    """
    def complete_tours(self, user):
        """
        Marks any completed tours as complete
        """
        if not user.pk:
            return None
        tours = self.filter(tourstatus__user=user, tourstatus__complete=False)
        for tour in tours:
            tour_class = tour.load_tour_class()
            if tour_class.is_complete(user):
                tour_class.mark_complete(user)

    def get_for_user(self, user):
        """
        Checks if a tour exists for a user and returns the tour instance
        """
        if not user.pk:
            return None
        self.complete_tours(user)
        return self.filter(tourstatus__user=user, tourstatus__complete=False).first()

    def get_recent_tour(self, user):
        if not user.pk:
            return None
        return self.filter(tourstatus__user=user).order_by(
            'tourstatus__complete', '-tourstatus__complete_time').first()

    def get_next_url(self, user):
        """
        Convenience method to get the next url for the specified user
        """
        if not user.pk:
            return None
        tour = self.get_for_user(user)
        if not tour:
            tour = self.get_recent_tour(user)
        if tour:
            return tour.load_tour_class().get_next_url(user)
        return None


@six.python_2_unicode_compatible
class Tour(models.Model):
    """
    Container object for tour steps. Provides functionality for loading the tour logic class
    and fetching the steps in the correct order.
    """
    name = models.CharField(max_length=128, unique=True)
    display_name = models.CharField(max_length=128)
    tour_class = models.CharField(max_length=128, unique=True)
    users = models.ManyToManyField(settings.AUTH_USER_MODEL, through='TourStatus')
    complete_url = models.CharField(max_length=128, blank=True, null=True, default=None)

    objects = TourManager()

    def load_tour_class(self):
        """
        Imports and returns the tour class.
        :return: The tour class instance determined by `tour_class`
        :rtype: BaseTour
        """
        return import_by_path(self.tour_class)(self)

    def __str__(self):
        return '{0}'.format(self.display_name)


@six.python_2_unicode_compatible
class Step(models.Model):
    """
    Represents one step of the tour that must be completed. The custom logic is implemented
    in the class specified in step_class
    """
    name = models.CharField(max_length=128, unique=True)
    display_name = models.CharField(max_length=128)
    url = models.CharField(max_length=128, null=True, blank=True)
    tour = models.ForeignKey(Tour, related_name='steps')
    parent_step = models.ForeignKey('self', null=True, related_name='steps')
    step_class = models.CharField(max_length=128, unique=True)
    sort_order = models.IntegerField(default=0)

    objects = ManagerUtilsManager()

    def load_step_class(self):
        """
        Imports and returns the step class.
        """
        return import_by_path(self.step_class)(self)

    def __str__(self):
        return '{0}'.format(self.display_name)


class TourStatus(models.Model):
    """
    This is the model that represents the relationship between a user and a tour. Keeps
    track of whether the tour has been completed by a user.
    """
    tour = models.ForeignKey(Tour)
    user = models.ForeignKey(settings.AUTH_USER_MODEL)
    complete = models.BooleanField(default=False)
    create_time = models.DateTimeField(auto_now_add=True)
    complete_time = models.DateTimeField(null=True, blank=True, default=None)

    objects = ManagerUtilsManager()



from rest_framework import serializers
from tour.models import Tour, Step


class TourSerializer(serializers.ModelSerializer):
    steps = serializers.SerializerMethodField('get_steps')

    class Meta:
        model = Tour
        fields = ('name', 'display_name', 'complete_url', 'steps')

    def get_steps(self, tour):
        return [
            StepSerializer(child_step, context=self.context).data
            for child_step in tour.load_tour_class().get_steps(0)
        ]


class StepSerializer(serializers.ModelSerializer):
    steps = serializers.SerializerMethodField('get_steps')
    complete = serializers.SerializerMethodField('get_complete')

    class Meta:
        model = Step
        fields = ('name', 'display_name', 'url', 'sort_order', 'steps', 'complete')

    def get_steps(self, step):
        return [
            StepSerializer(child_step, context=self.context).data
            for child_step in step.load_step_class().get_steps(0)
        ]

    def get_complete(self, step):
        if 'request' in self.context:
            return step.load_step_class().is_complete(self.context['request'].user)
        return False



from django.contrib.auth.models import User
from django.test import TestCase
from django_dynamic_fixture import G
from mock import patch
from tour.models import Tour, TourStatus, Step
from tour.tests.tour_tests import BaseTourTest


class TourManagerTest(BaseTourTest):
    """
    Tests the methods of the TourManager
    """
    @patch('tour.tests.mocks.MockStep1.is_complete', spec_set=True)
    def test_complete_tours(self, mock_step1_is_complete):
        """
        Verifies that any completed tours get marked as complete
        :type mock_step1_is_complete: Mock
        """
        mock_step1_is_complete.return_value = False
        self.tour1.steps.add(self.step1)

        # add user to tours
        self.tour1.load_tour_class().add_user(self.test_user)
        self.tour2.load_tour_class().add_user(self.test_user)

        Tour.objects.complete_tours(self.test_user)
        self.assertEqual(1, TourStatus.objects.filter(complete=True).count())
        self.assertEqual(1, TourStatus.objects.filter(complete=False).count())

    def test_complete_tours_no_user(self):
        """
        Makes sure None is returned if the user is anonymous
        """
        self.assertIsNone(Tour.objects.complete_tours(User()))

    @patch('tour.tests.mocks.MockStep1.is_complete', spec_set=True)
    def test_get_for_user(self, mock_step1_is_complete):
        """
        Verifies that an incomplete tour class is fetched for a user
        :type mock_step1_is_complete: Mock
        """
        mock_step1_is_complete.return_value = False
        self.tour1.steps.add(self.step1)
        self.tour2.steps.add(self.step2)

        # add users to tours
        self.tour1.load_tour_class().add_user(self.test_user)
        self.tour1.load_tour_class().add_user(self.test_user2)
        self.tour2.load_tour_class().add_user(self.test_user)
        self.tour2.load_tour_class().add_user(self.test_user2)

        # tour 2 will be completed, so check for tour 1
        self.assertEqual(self.tour1, Tour.objects.get_for_user(self.test_user))

    def test_get_for_user_no_user(self):
        """
        Makes sure None is returned if the user is anonymous
        """
        self.assertIsNone(Tour.objects.get_for_user(User()))

    def test_get_for_user_empty(self):
        """
        Verifies that None is returned if no incomplete tours
        """
        self.tour1.steps.add(self.step1)

        # add user to tour
        self.tour1.load_tour_class().add_user(self.test_user)
        self.tour1.load_tour_class().add_user(self.test_user2)
        self.tour2.load_tour_class().add_user(self.test_user)
        self.tour2.load_tour_class().add_user(self.test_user2)

        # all tours should be completed
        self.assertIsNone(Tour.objects.get_for_user(self.test_user))

    def test_get_recent_tour_complete(self):
        """
        Makes sure that a recently completed tour is returned. Verifies most recently completed tour
        is returned
        """
        # add user to tour
        self.tour1.load_tour_class().add_user(self.test_user)
        self.tour1.load_tour_class().add_user(self.test_user2)
        self.tour2.load_tour_class().add_user(self.test_user)
        self.tour2.load_tour_class().add_user(self.test_user2)

        # complete tours
        self.tour2.load_tour_class().mark_complete(self.test_user)
        self.tour1.load_tour_class().mark_complete(self.test_user)
        self.tour1.load_tour_class().mark_complete(self.test_user2)

        # make sure complete
        self.assertEqual(3, TourStatus.objects.filter(complete=True).count())

        # check that correct tour is returned
        self.assertEqual(self.tour1, Tour.objects.get_recent_tour(self.test_user))

    def test_get_recent_tour_no_user(self):
        """
        Makes sure None is returned if the user is anonymous
        """
        self.assertIsNone(Tour.objects.get_recent_tour(User()))

    def test_get_recent_tour_incomplete(self):
        """
        Makes sure that an incomplete tour is returned before a complete tour
        """
        # add user to tour
        self.tour1.load_tour_class().add_user(self.test_user)
        self.tour1.load_tour_class().add_user(self.test_user2)
        self.tour2.load_tour_class().add_user(self.test_user)
        self.tour2.load_tour_class().add_user(self.test_user2)

        # complete tours
        self.tour1.load_tour_class().mark_complete(self.test_user)
        self.tour2.load_tour_class().mark_complete(self.test_user)
        self.tour1.load_tour_class().mark_complete(self.test_user2)

        # make sure complete
        self.assertEqual(3, TourStatus.objects.filter(complete=True).count())

        # check that correct tour is returned
        self.assertEqual(self.tour2, Tour.objects.get_recent_tour(self.test_user2))

    def test_get_recent_tour_none(self):
        """
        Makes sure None is returned if there is no completed tour
        """
        self.assertIsNone(Tour.objects.get_recent_tour(self.test_user))

    @patch('tour.tests.mocks.MockStep1.is_complete', spec_set=True)
    @patch('tour.models.TourManager.get_for_user')
    def test_get_next_url_current_tour(self, mock_get_for_user, mock_step1_is_complete):
        """
        Verifies that the next url of a tour will be returned
        :type mock_get_for_user: Mock
        :type mock_step1_is_complete: Mock
        """
        mock_get_for_user.return_value = self.tour1
        mock_step1_is_complete.return_value = False

        self.tour1.steps.add(self.step1)

        # add user to tour
        self.tour1.load_tour_class().add_user(self.test_user)
        self.tour1.load_tour_class().add_user(self.test_user2)
        self.tour2.load_tour_class().add_user(self.test_user)
        self.tour2.load_tour_class().add_user(self.test_user2)

        self.assertEqual(self.step1.url, Tour.objects.get_next_url(self.test_user))
        self.assertEqual(1, mock_get_for_user.call_count)

    @patch('tour.tests.mocks.MockStep1.is_complete', spec_set=True)
    @patch('tour.models.TourManager.get_recent_tour')
    @patch('tour.models.TourManager.get_for_user')
    def test_get_next_url_recent_tour(self, mock_get_for_user, mock_get_recent_tour, mock_step1_is_complete):
        """
        Verifies that the next url of a recent tour will be returned
        :type mock_get_for_user: Mock
        :type mock_get_recent_tour: Mock
        :type mock_step1_is_complete: Mock
        """
        mock_get_for_user.return_value = None
        mock_get_recent_tour.return_value = self.tour2
        mock_step1_is_complete.return_value = False

        self.tour2.steps.add(self.step1)

        # add user to tour
        self.tour1.load_tour_class().add_user(self.test_user)
        self.tour1.load_tour_class().add_user(self.test_user2)
        self.tour2.load_tour_class().add_user(self.test_user)
        self.tour2.load_tour_class().add_user(self.test_user2)

        self.assertEqual(self.step1.url, Tour.objects.get_next_url(self.test_user))
        self.assertEqual(1, mock_get_for_user.call_count)
        self.assertEqual(1, mock_get_recent_tour.call_count)

    def test_get_next_url_no_user(self):
        """
        Makes sure None is returned if the user is anonymous
        """
        self.assertIsNone(Tour.objects.get_next_url(User()))

    def test_get_next_url_none(self):
        """
        Verifies that None is returned if no tours exist for a user
        """
        self.assertIsNone(Tour.objects.get_next_url(self.test_user))


class TourTest(TestCase):

    def test_str(self):
        """
        Tests the tour str method
        """
        tour = G(Tour, display_name='test1')
        self.assertEqual('test1', str(tour))


class StepTest(TestCase):

    def test_str(self):
        """
        Tests the step str method
        """
        step = G(Step, display_name='test1')
        self.assertEqual('test1', str(step))



from django.contrib.auth.models import User
from django.template import Template, Context
from mock import Mock, patch

from tour.tests.tour_tests import BaseTourTest


class TemplateTagTest(BaseTourTest):
    """
    Tests the functionality of the template tag
    """

    def setUp(self):
        super(TemplateTagTest, self).setUp()
        self.test_template = Template('{% load tour_tags %}{% tour_navigation %}')
        self.tour1.steps.add(self.step1, self.step2, self.step3, self.step4)

    def test_template_no_user(self):
        """
        Verifies that the tour template does not get rendered without a user
        """
        context = Context({
            'request': Mock(
                user=User(),
                path='/mock/path',
                method='get',
                GET={},
            ),
        })
        self.assertEqual('', self.test_template.render(context))

    def test_template_no_tour(self):
        """
        Verifies that the tour template does not get rendered if a user doesn't have a tour
        """
        self.login_user1()
        context = Context({
            'request': Mock(
                user=self.test_user,
                path='/mock/path',
                method='get',
                GET={},
            ),
        })
        self.assertEqual('', self.test_template.render(context).strip())

    @patch('tour.tests.mocks.MockStep1.is_complete', spec_set=True)
    def test_user_with_tour(self, mock_step1_is_complete):
        """
        Verifies that the tour template gets rendered if a user has a tour
        :type mock_step1_is_complete: Mock
        """
        mock_step1_is_complete.return_value = False

        self.login_user1()
        self.tour1.load_tour_class().add_user(self.test_user)
        context = Context({
            'request': Mock(
                user=self.test_user,
                path='/mock/path',
                method='get',
                GET={},
            ),
        })
        self.assertTrue('tour-wrap' in self.test_template.render(context))

    @patch('tour.tests.mocks.MockStep1.is_complete', spec_set=True)
    def test_current_class(self, mock_step1_is_complete):
        """
        Verify that the current class gets applied
        :type mock_step1_is_complete: Mock
        """
        mock_step1_is_complete.return_value = False

        self.login_user1()
        self.tour1.load_tour_class().add_user(self.test_user)
        context = Context({
            'request': Mock(
                user=self.test_user,
                path='mock1',
                method='get',
                GET={},
            ),
        })
        self.assertTrue('current' in self.test_template.render(context))

    @patch('tour.tests.mocks.MockStep2.is_complete', spec_set=True)
    def test_complete_class(self, mock_step2_is_complete):
        """
        Verify that the complete class gets applied
        :type mock_step2_is_complete: Mock
        """
        mock_step2_is_complete.return_value = False

        self.login_user1()
        self.tour1.load_tour_class().add_user(self.test_user)
        context = Context({
            'request': Mock(
                user=self.test_user,
                path='/mock/path',
                method='get',
                GET={},
            ),
        })
        self.assertTrue('complete' in self.test_template.render(context))

    def test_complete_tour(self):
        """
        Make sure no tour gets rendered when it is complete
        """
        self.login_user1()
        self.tour1.load_tour_class().add_user(self.test_user)
        context = Context({
            'request': Mock(
                user=self.test_user,
                path='/mock/path',
                method='get',
                GET={},
            ),
        })
        self.assertTrue('tour-wrap' not in self.test_template.render(context))

    def test_always_display(self):
        """
        Makes sure that the tour does get displayed if the always_show flag is on
        """
        self.test_template = Template('{% load tour_tags %}{% tour_navigation always_show=True %}')

        self.login_user1()
        self.tour1.load_tour_class().add_user(self.test_user)
        context = Context({
            'request': Mock(
                user=self.test_user,
                path='/mock/path',
                method='get',
                GET={},
            ),
        })
        self.assertTrue('tour-wrap' in self.test_template.render(context))

    def test_missing_request(self):
        """
        Verify no errors for missing request object
        """
        context = Context({})
        self.assertEqual('', self.test_template.render(context))

    @patch('tour.tests.mocks.MockStep2.is_complete', spec_set=True)
    @patch('tour.tests.mocks.MockStep1.is_complete', spec_set=True)
    def test_step_classes(self, mock_step1_is_complete, mock_step2_is_complete):
        """
        Test that the second step has an available class but not a complete class
        :type mock_step1_is_complete: Mock
        :type mock_step2_is_complete: Mock
        """
        mock_step1_is_complete.return_value = True
        mock_step2_is_complete.return_value = False
        self.test_template = Template('{% load tour_tags %}{% tour_navigation always_show=True %}')

        self.login_user1()
        self.tour1.load_tour_class().add_user(self.test_user)
        context = Context({
            'request': Mock(
                user=self.test_user,
                path='/mock/path',
                method='get',
                GET={},
            ),
        })
        rendered_content = self.render_and_clean(self.test_template, context)

        # test incomplete unavailable
        expected_str = '<a href="#" class="step-circle incomplete unavailable">'
        self.assertTrue(expected_str in rendered_content)

        # test incomplete available
        expected_str = '<a href="mock2" class="step-circle incomplete available">'
        self.assertTrue(expected_str in rendered_content)

        # complete the second step
        mock_step2_is_complete.return_value = True

        # request the first page
        context = Context({
            'request': Mock(
                user=self.test_user,
                path='mock1',
                method='get',
                GET={},
            ),
        })
        rendered_content = self.render_and_clean(self.test_template, context)

        # test available
        expected_str = '<a href="mock2" class="step-circle available">'
        self.assertTrue(expected_str in rendered_content)

        # test current complete available
        expected_str = '<a href="mock1" class="step-circle current available">'
        self.assertTrue(expected_str in rendered_content)

    @patch('tour.tests.mocks.MockStep1.is_complete', spec_set=True)
    def test_step_display_name(self, mock_step1_is_complete):
        """
        Makes sure the appropriate title gets displayed for the tour title
        :type mock_step1_is_complete: Mock
        """
        mock_step1_is_complete.return_value = False

        self.login_user1()
        self.tour1.load_tour_class().add_user(self.test_user)
        context = Context({
            'request': Mock(
                user=self.test_user,
                path='mock1',
                method='get',
                GET={},
            ),
        })
        rendered_content = self.render_and_clean(self.test_template, context)

        # Make sure the current step is displayed
        expected_html = '<div class="tour-name">{0}</div>'.format(self.step1.display_name)
        self.assertTrue(expected_html in rendered_content)

    @patch('tour.tests.mocks.MockStep1.is_complete', spec_set=True)
    def test_tour_display_name(self, mock_step1_is_complete):
        """
        Makes sure the appropriate title gets displayed for the tour title
        :type mock_step1_is_complete: Mock
        """
        mock_step1_is_complete.return_value = False

        self.login_user1()
        self.tour1.load_tour_class().add_user(self.test_user)

        # Make sure the tour title is displayed
        context = Context({
            'request': Mock(
                user=self.test_user,
                path='mock0',
                method='get',
                GET={},
            ),
        })
        rendered_content = self.render_and_clean(self.test_template, context)
        expected_html = '<div class="tour-name">{0}</div>'.format(self.tour1.display_name)
        self.assertTrue(expected_html in rendered_content)

    def render_and_clean(self, template, context):
        # render the template
        rendered_content = template.render(context).strip()
        # remove tabs
        rendered_content = rendered_content.replace('    ', '')
        # remove new lines
        rendered_content = rendered_content.replace('\n', '')
        return rendered_content



from django.http import HttpResponse
from django.views.generic import View
from tour.tours import BaseStep, BaseTour
from tour.views import TourStepMixin


mock_null_value = None


class MockView(TourStepMixin, View):
    def get(self, request):
        return HttpResponse('ok')


class MockStep1(BaseStep):
    pass


class MockStep2(BaseStep):
    pass


class MockStep3(BaseStep):
    pass


class MockStep4(BaseStep):
    pass


class MockTour(BaseTour):
    pass


class MockTour2(BaseTour):
    pass



from mock import Mock, patch
from tour.tests.mocks import MockView
from tour.tests.tour_tests import BaseTourTest


class ViewTest(BaseTourTest):
    """
    Tests the functionality of all tour views and mixins
    """
    def setUp(self):
        super(ViewTest, self).setUp()
        self.tour1.steps.add(self.step1, self.step2, self.step3)

    def test_no_tour(self):
        """
        Verify that the user isn't redirected with no tour
        """
        mock_request = Mock(user=self.test_user, path='mock2', method='get', GET={})
        mock_view = MockView(request=mock_request)
        response = mock_view.dispatch(mock_request)
        self.assertEqual(200, response.status_code)

    @patch('tour.tests.mocks.MockStep1.is_complete', spec_set=True)
    def test_request_unrelated_page(self, mock_step1_is_complete):
        """
        Verifies that user can go to a non-tour page
        :type mock_step1_is_complete: Mock
        """
        mock_step1_is_complete.return_value = False

        self.tour1.load_tour_class().add_user(self.test_user)

        # request page that isn't in the tour before tour is complete
        mock_request = Mock(user=self.test_user, path='mock-fake', method='get', GET={})
        mock_view = MockView(request=mock_request)
        response = mock_view.dispatch(mock_request)
        self.assertEqual(200, response.status_code)

    @patch('tour.tests.mocks.MockStep1.is_complete', spec_set=True)
    def test_request_first_step(self, mock_step1_is_complete):
        """
        Request the first step of the tour and verify that the page is accessible
        :type mock_step1_is_complete: Mock
        """
        mock_step1_is_complete.return_value = False

        self.tour1.load_tour_class().add_user(self.test_user)
        mock_request = Mock(user=self.test_user, path='mock1', method='get', GET={})
        mock_view = MockView(request=mock_request)
        response = mock_view.dispatch(mock_request)
        self.assertEqual(200, response.status_code)

    @patch('tour.tests.mocks.MockStep1.is_complete', spec_set=True)
    def test_redirect_from_future_step(self, mock_step1_is_complete):
        """
        Try to access the second step of the tour before the first is complete. The user should be
        redirected to the first step
        :type mock_step1_is_complete: Mock
        """
        mock_step1_is_complete.return_value = False

        # do request to second step when we should be on first
        self.tour1.load_tour_class().add_user(self.test_user)
        mock_request = Mock(user=self.test_user, path='mock2', method='get', GET={})
        mock_view = MockView(request=mock_request)
        response = mock_view.dispatch(mock_request)
        self.assertEqual(302, response.status_code)
        self.assertEqual('mock1', response.url)

    def test_tour_complete_url_redirect(self):
        """
        Verifies that a user can't go to steps out of order and can't go to other steps
        after the tour is complete
        """
        # complete tour and try to go to first step
        self.tour1.load_tour_class().add_user(self.test_user)
        mock_request = Mock(user=self.test_user, path='mock1', method='get', GET={})
        mock_view = MockView(request=mock_request)
        response = mock_view.dispatch(mock_request)
        self.assertEqual(302, response.status_code)
        self.assertEqual('mock_complete1', response.url)

    def test_tour_complete_unrelated_page(self):
        """
        Verifies that the user can access a non-tour page after finishing a tour
        """
        # request page that isn't in the tour when tour is complete
        self.tour1.load_tour_class().add_user(self.test_user)
        mock_request = Mock(user=self.test_user, path='mock-fake', method='get', GET={})
        mock_view = MockView(request=mock_request)
        response = mock_view.dispatch(mock_request)
        self.assertEqual(200, response.status_code)



from tour.serializers import TourSerializer, StepSerializer
from tour.tests.tour_tests import BaseTourTest


class SerializerTest(BaseTourTest):
    """
    Tests the serializers contained in the tour app
    """
    def test_step_serializer(self):
        """
        Tests the Step model serialization
        """
        self.tour1.steps.add(self.step1, self.step2)
        self.step1.steps.add(self.step3, self.step4)
        self.assertEqual(StepSerializer(self.step1).data, {
            'name': 'mock1',
            'display_name': 'Mock Step 1',
            'url': 'mock1',
            'sort_order': 0,
            'complete': False,
            'steps': [{
                'name': 'mock3',
                'display_name': 'Mock Step 3',
                'url': 'mock3',
                'sort_order': 2,
                'steps': [],
                'complete': False,
            }, {
                'name': 'mock4',
                'display_name': 'Mock Step 4',
                'url': 'mock4',
                'sort_order': 3,
                'steps': [],
                'complete': False,
            }]
        })

    def test_tour_serializer(self):
        """
        Tests the Tour model serialization
        """
        self.tour1.steps.add(self.step1, self.step2)
        self.step1.steps.add(self.step3, self.step4)
        self.assertEqual(TourSerializer(self.tour1).data, {
            'name': 'tour1',
            'display_name': 'Mock Tour',
            'complete_url': 'mock_complete1',
            'steps': [{
                'name': 'mock1',
                'display_name': 'Mock Step 1',
                'url': 'mock1',
                'sort_order': 0,
                'complete': False,
                'steps': [{
                    'name': 'mock3',
                    'display_name': 'Mock Step 3',
                    'url': 'mock3',
                    'sort_order': 2,
                    'steps': [],
                    'complete': False,
                }, {
                    'name': 'mock4',
                    'display_name': 'Mock Step 4',
                    'url': 'mock4',
                    'sort_order': 3,
                    'steps': [],
                    'complete': False,
                }]
            }, {
                'name': 'mock2',
                'display_name': 'Mock Step 2',
                'url': 'mock2',
                'sort_order': 1,
                'steps': [],
                'complete': False,
            }]
        })



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Step',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', primary_key=True, auto_created=True)),
                ('name', models.CharField(max_length=128, unique=True)),
                ('display_name', models.CharField(max_length=128)),
                ('url', models.CharField(max_length=128, blank=True, null=True)),
                ('step_class', models.CharField(max_length=128, unique=True)),
                ('sort_order', models.IntegerField(default=0)),
                ('parent_step', models.ForeignKey(related_name='steps', null=True, to='tour.Step')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Tour',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', primary_key=True, auto_created=True)),
                ('name', models.CharField(max_length=128, unique=True)),
                ('display_name', models.CharField(max_length=128)),
                ('tour_class', models.CharField(max_length=128, unique=True)),
                ('complete_url', models.CharField(max_length=128, blank=True, null=True, default=None)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='TourStatus',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', primary_key=True, auto_created=True)),
                ('complete', models.BooleanField(default=False)),
                ('create_time', models.DateTimeField(auto_now_add=True)),
                ('complete_time', models.DateTimeField(blank=True, null=True, default=None)),
                ('tour', models.ForeignKey(to='tour.Tour')),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='tour',
            name='users',
            field=models.ManyToManyField(to=settings.AUTH_USER_MODEL, through='tour.TourStatus'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='step',
            name='tour',
            field=models.ForeignKey(to='tour.Tour', related_name='steps'),
            preserve_default=True,
        ),
    ]



from django.contrib.auth.models import User
from django.test import TestCase
from django_dynamic_fixture import G
from mock import patch

from tour.models import Tour, Step, TourStatus


class BaseTourTest(TestCase):
    """
    Provides basic setup for tour tests like creating users
    """
    def setUp(self):
        super(BaseTourTest, self).setUp()

        self.test_user = User.objects.create_user('test', 'test@gmail.com', 'test')
        self.test_user2 = User.objects.create_user('test2', 'test2@gmail.com', 'test2')

        self.tour1 = G(
            Tour, display_name='Mock Tour', name='tour1', complete_url='mock_complete1',
            tour_class='tour.tests.mocks.MockTour')
        self.tour2 = G(
            Tour, display_name='Mock Tour 2', name='tour2', complete_url='mock_complete2',
            tour_class='tour.tests.mocks.MockTour2')
        self.step1 = G(
            Step, step_class='tour.tests.mocks.MockStep1', display_name='Mock Step 1', name='mock1',
            url='mock1', parent_step=None, sort_order=0)
        self.step2 = G(
            Step, step_class='tour.tests.mocks.MockStep2', display_name='Mock Step 2', name='mock2',
            url='mock2', parent_step=None, sort_order=1)
        self.step3 = G(
            Step, step_class='tour.tests.mocks.MockStep3', display_name='Mock Step 3', name='mock3',
            url='mock3', parent_step=None, sort_order=2)
        self.step4 = G(
            Step, step_class='tour.tests.mocks.MockStep4', display_name='Mock Step 4', name='mock4',
            url='mock4', parent_step=None, sort_order=3)
        self.step5 = G(
            Step, step_class='tour.tours.BaseStep', display_name='Mock Step 5', name='mock5',
            url=None, parent_step=None, sort_order=4)

    def login_user1(self):
        self.client.login(username='test', password='test')


class TourTest(BaseTourTest):
    """
    Tests the functionality of the BaseTour class
    """
    def test_init(self):
        """
        Verifies that the tour object is properly set when loaded
        """
        self.assertEqual(self.tour1.load_tour_class().tour, self.tour1)

    def test_get_steps_flat(self):
        """
        Verifies that the steps are loaded in the correct order
        """
        self.step1.sort_order = 1
        self.step1.save()
        self.step2.sort_order = 0
        self.step2.save()

        self.tour1.steps.add(self.step1, self.step2)
        expected_steps = [self.step2, self.step1]
        self.assertEqual(expected_steps, self.tour1.load_tour_class().get_steps())

    def test_get_steps_nested(self):
        """
        Verifies that the nested steps are loaded correctly
        """
        self.tour1.steps.add(self.step1, self.step2)
        self.step1.steps.add(self.step3, self.step4)

        self.step3.sort_order = 1
        self.step3.save()
        self.step4.sort_order = 0
        self.step4.save()

        expected_steps = [self.step1, self.step4, self.step3, self.step2]
        self.assertEqual(expected_steps, self.tour1.load_tour_class().get_steps())

    def test_get_url_list(self):
        """
        Verifies that the tour returns the correct step url list
        """
        self.tour1.steps.add(self.step1, self.step5, self.step2)
        expected_url_list = ['mock1', 'mock2']
        self.assertEqual(expected_url_list, self.tour1.load_tour_class().get_url_list())

    def test_add_user(self):
        """
        Verifies that a user is linked to a tour properly and that the correct tour is returned
        """
        # add user to tour
        tour_status = self.tour1.load_tour_class().add_user(self.test_user)

        # try to add again and make sure it returns the same status
        self.assertEqual(tour_status, self.tour1.load_tour_class().add_user(self.test_user))

        # make sure only one status
        self.assertEqual(1, TourStatus.objects.count())

        # mark status as complete
        tour_status.complete = True
        tour_status.save()

        # make sure another tour is created
        self.tour1.load_tour_class().add_user(self.test_user)
        self.assertEqual(2, TourStatus.objects.count())
        self.assertEqual(1, TourStatus.objects.filter(complete=False).count())

    def test_mark_complete(self):
        """
        Verifies that a tour status record will be marked as complete for a user
        """
        # add multiple users to multiple tours
        tour1_class = self.tour1.load_tour_class()
        tour2_class = self.tour2.load_tour_class()
        tour1_class.add_user(self.test_user)
        tour1_class.add_user(self.test_user2)
        tour2_class.add_user(self.test_user)
        tour2_class.add_user(self.test_user2)

        # make sure there are 4 records
        self.assertEqual(4, TourStatus.objects.count())
        # complete the tour for user1
        self.assertTrue(tour1_class.mark_complete(self.test_user))
        # make sure it is complete
        self.assertEqual(1, TourStatus.objects.filter(complete=True).count())
        # try to complete the same tour
        self.assertFalse(tour1_class.mark_complete(self.test_user))
        # add the user to the tour again
        tour1_class.add_user(self.test_user)
        # make sure there are 5 records
        self.assertEqual(5, TourStatus.objects.count())

    @patch('tour.tests.mocks.MockStep4.is_complete', spec_set=True)
    @patch('tour.tests.mocks.MockStep3.is_complete', spec_set=True)
    @patch('tour.tests.mocks.MockStep2.is_complete', spec_set=True)
    @patch('tour.tests.mocks.MockStep1.is_complete', spec_set=True)
    def test_get_current_step(
            self, mock_step1_is_complete, mock_step2_is_complete, mock_step3_is_complete, mock_step4_is_complete):
        """
        Verifies that the tour class returns the first incomplete step
        :type mock_step1_is_complete: Mock
        :type mock_step2_is_complete: Mock
        :type mock_step3_is_complete: Mock
        :type mock_step4_is_complete: Mock
        """
        mock_step1_is_complete.return_value = False
        mock_step2_is_complete.return_value = False
        mock_step3_is_complete.return_value = False
        mock_step4_is_complete.return_value = False

        self.tour1.steps.add(self.step1, self.step2)
        self.step1.steps.add(self.step3, self.step4)
        tour1_class = self.tour1.load_tour_class()

        self.assertEqual(self.step1, tour1_class.get_current_step(self.test_user))

        mock_step1_is_complete.return_value = True
        mock_step3_is_complete.return_value = True
        self.assertEqual(self.step4, tour1_class.get_current_step(self.test_user))

        mock_step4_is_complete.return_value = True
        mock_step2_is_complete.return_value = True
        self.assertIsNone(tour1_class.get_current_step(self.test_user))

    @patch('tour.tests.mocks.MockStep4.is_complete', spec_set=True)
    @patch('tour.tests.mocks.MockStep3.is_complete', spec_set=True)
    @patch('tour.tests.mocks.MockStep2.is_complete', spec_set=True)
    @patch('tour.tests.mocks.MockStep1.is_complete', spec_set=True)
    def test_get_next_url(
            self, mock_step1_is_complete, mock_step2_is_complete, mock_step3_is_complete, mock_step4_is_complete):
        """
        Verifies that the url is returned for the current step
        :type mock_step1_is_complete: Mock
        :type mock_step2_is_complete: Mock
        :type mock_step3_is_complete: Mock
        :type mock_step4_is_complete: Mock
        """
        mock_step1_is_complete.return_value = False
        mock_step2_is_complete.return_value = False
        mock_step3_is_complete.return_value = False
        mock_step4_is_complete.return_value = False

        self.step5.sort_order = 1
        self.step5.save()
        self.step2.sort_order = 3
        self.step2.save()

        self.tour1.steps.add(self.step1, self.step2, self.step5)
        self.step5.steps.add(self.step3, self.step4)
        tour1_class = self.tour1.load_tour_class()

        self.assertEqual('mock1', tour1_class.get_next_url(self.test_user))

        mock_step1_is_complete.return_value = True
        self.assertEqual('mock3', tour1_class.get_next_url(self.test_user))

        mock_step3_is_complete.return_value = True
        self.assertEqual('mock4', tour1_class.get_next_url(self.test_user))

        mock_step4_is_complete.return_value = True
        self.assertEqual('mock2', tour1_class.get_next_url(self.test_user))

        mock_step2_is_complete.return_value = True
        self.assertEqual('mock_complete1', tour1_class.get_next_url(self.test_user))

    @patch('tour.tests.mocks.MockStep1.is_complete', spec_set=True)
    def test_is_complete(self, mock_step1_is_complete):
        """
        Verifies that a tour returns true when complete and false when incomplete
        :type mock_step1_is_complete: Mock
        """
        mock_step1_is_complete.return_value = False

        self.tour1.steps.add(self.step1)
        tour1_class = self.tour1.load_tour_class()

        self.assertFalse(tour1_class.is_complete(self.test_user))

        mock_step1_is_complete.return_value = True
        self.assertTrue(tour1_class.is_complete(self.test_user))


class StepTest(BaseTourTest):
    """
    Tests the functionality of the BaseStep class
    """
    def test_init(self):
        """
        Verifies that the step object is properly set when loaded
        """
        self.assertEqual(self.step1.load_step_class().step, self.step1)

    def test_is_complete(self):
        """
        Verifies that a step returns true by default
        """
        step1_class = self.step1.load_step_class()
        self.assertTrue(step1_class.is_complete(self.test_user))

    def test_get_steps_flat(self):
        """
        Verifies that the steps are loaded in the correct order
        """
        self.step1.steps.add(self.step2, self.step3)
        expected_steps = [self.step2, self.step3]
        self.assertEqual(expected_steps, self.step1.load_step_class().get_steps())

    def test_get_steps_nested(self):
        """
        Verifies that the nested steps are loaded correctly
        """
        self.step1.steps.add(self.step2)
        self.step2.steps.add(self.step3, self.step4)
        expected_steps = [self.step2, self.step3, self.step4]
        self.assertEqual(expected_steps, self.step1.load_step_class().get_steps())



from copy import deepcopy

from django import template
from django.template.loader import get_template

from tour.models import Tour
from tour.serializers import TourSerializer


register = template.Library()


class TourNavNode(template.Node):
    def __init__(self, always_show=False):
        self.always_show = always_show

    def get_tour(self, request):
        # Check for any tours
        tour = Tour.objects.get_for_user(request.user)

        if not tour and self.always_show:
            tour = Tour.objects.get_recent_tour(request.user)
        if self.always_show:
            mutable_get = deepcopy(request.GET)
            mutable_get['always_show'] = True
            request.GET = mutable_get
        return tour

    def get_tour_dict(self, tour, context):
        if not tour:
            return None

        tour_dict = TourSerializer(tour, context=context).data

        # Set the step css classes
        previous_steps_complete = True
        is_after_current = False
        for step_dict in tour_dict['steps']:
            classes = []
            if step_dict['url'] == context['request'].path:
                classes.append('current')
                step_dict['current'] = True
                tour_dict['display_name'] = step_dict['display_name']
                is_after_current = True
            if not previous_steps_complete:
                classes.append('incomplete')
                classes.append('unavailable')
                step_dict['url'] = '#'
            elif not step_dict['complete']:
                classes.append('incomplete')
                classes.append('available')
                previous_steps_complete = False
            elif is_after_current:
                classes.append('available')
            else:
                classes.append('complete')
                classes.append('available')
            step_dict['cls'] = ' '.join(classes)

        return tour_dict

    def render(self, context):
        if 'request' in context and hasattr(context['request'], 'user'):
            # Make sure this isn't the anonymous user
            if not context['request'].user.id:
                return ''

            tour = self.get_tour(context['request'])
            context['tour'] = self.get_tour_dict(tour, context)

            # Load the tour template and render it
            tour_template = get_template('tour/tour_navigation.html')
            return tour_template.render(context)
        return ''


@register.simple_tag(takes_context=True)
def tour_navigation(context, **kwargs):
    """
    Tag to render the tour nav node
    """
    return TourNavNode(always_show=kwargs.get('always_show', False)).render(context)



import datetime

from tour.models import TourStatus


class BaseStep(object):
    """
    Base step class that handles the creation of step records and determines when the step is complete
    """
    def __init__(self, step):
        self.step = step

    def is_complete(self, user=None):
        """
        This is meant to be implemented in subclasses. This checks conditions to determine if the step is complete
        """
        return True

    def get_steps(self, depth=-1):
        """
        Returns the steps in order based on if there is a parent or not
        TODO: optimize this
        """
        all_steps = []
        steps = self.step.steps.filter(parent_step=self.step).order_by('sort_order')
        for step in steps:
            all_steps.append(step)
            if depth != 0:
                all_steps.extend(step.load_step_class().get_steps(depth=depth - 1))
        return all_steps


class BaseTour(object):
    """
    Base tour class that handles the creation of tour records and determines when tours are complete.
    """
    def __init__(self, tour):
        # self.current_step_class = None
        self.tour = tour

    def get_steps(self, depth=-1):
        """
        Returns the steps in order based on if there is a parent or not
        TODO: optimize this
        """
        all_steps = []
        steps = self.tour.steps.filter(parent_step=None).order_by('sort_order')
        for step in steps:
            all_steps.append(step)
            if depth != 0:
                all_steps.extend(step.load_step_class().get_steps(depth=depth - 1))
        return all_steps

    def get_url_list(self):
        """
        Returns a flattened list of urls of all steps contained in the tour.
        """
        return [step.url for step in self.get_steps() if step.url]

    def add_user(self, user):
        """
        Adds a relationship record for the user
        """
        instance, created = TourStatus.objects.get_or_create(tour=self.tour, user=user, complete=False)
        return instance

    def mark_complete(self, user):
        """
        Marks the tour status record as complete
        """
        tour_status = self.tour.tourstatus_set.all().filter(tour=self.tour, user=user, complete=False).first()
        if tour_status:
            tour_status.complete = True
            tour_status.complete_time = datetime.datetime.utcnow()
            tour_status.save()
            return True
        return False

    def get_current_step(self, user):
        """
        Finds the first incomplete steps and returns it
        :param user: The django user to find the current step for
        :type user: User
        :return: The first incomplete step
        :rtype: Step
        """
        for step in self.get_steps():
            if not step.load_step_class().is_complete(user):
                return step
        return None

    def get_next_url(self, user):
        """
        Gets the next url based on the current step.
        """
        current_step = self.get_current_step(user)
        return current_step.url if current_step else self.tour.complete_url

    def is_complete(self, user):
        """
        Checks the state of the steps to see if they are all complete
        """
        return False if self.get_current_step(user) else True



from rest_framework.authentication import SessionAuthentication
from rest_framework.generics import ListAPIView
from rest_framework.permissions import IsAuthenticated
from tour.filters import TourFilter
from tour.models import Tour
from tour.serializers import TourSerializer


class TourApiView(ListAPIView):
    serializer_class = TourSerializer
    filter_class = TourFilter
    permission_classes = (IsAuthenticated,)
    authentication_classes = (SessionAuthentication,)

    def get_queryset(self):
        Tour.objects.complete_tours(self.request.user)
        return Tour.objects.filter(tourstatus__user=self.request.user, tourstatus__complete=False)



from django.contrib.auth.models import User
from django.test import TestCase
from django_dynamic_fixture import G
from mock import Mock
from tour.models import Tour
from tour.api import TourApiView


class TourApiViewTest(TestCase):

    def test_get_queryset(self):
        """
        Should return a queryset for tours
        """
        view = TourApiView()
        view.request = Mock(user=G(User, id=1))
        self.assertEqual(Tour, view.get_queryset().model)



# import multiprocessing to avoid this bug (http://bugs.python.org/issue15881#msg170215)
import multiprocessing
assert multiprocessing
import re
from setuptools import setup, find_packages


def get_version():
    """
    Extracts the version number from the version.py file.
    """
    VERSION_FILE = 'tour/version.py'
    mo = re.search(r'^__version__ = [\'"]([^\'"]*)[\'"]', open(VERSION_FILE, 'rt').read(), re.M)
    if mo:
        return mo.group(1)
    else:
        raise RuntimeError('Unable to find version string in {0}.'.format(VERSION_FILE))


setup(
    name='django-tour',
    version=get_version(),
    description='Require the django user to complete a series of steps with custom logic',
    long_description=open('README.md').read(),
    url='https://github.com/ambitioninc/django-tour',
    author='Wes Okes',
    author_email='wes.okes@gmail.com',
    keywords='',
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Framework :: Django',
        'Framework :: Django :: 1.6',
        'Framework :: Django :: 1.7',
    ],
    license='MIT',
    install_requires=[
        'Django>=1.6',
        'djangorestframework>=2.3.13,<3.0',
        'django-manager-utils>=0.7.2',
        'django_filter>=0.7',
    ],
    tests_require=[
        'psycopg2',
        'django-nose',
        'mock',
        'south>=1.0.2',
        'django_dynamic_fixture',
    ],
    test_suite='run_tests.run_tests',
    include_package_data=True,
)
